﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

using System.Runtime.InteropServices;

namespace howto_skinned_form
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }

        // Prepare the form.
        private void Form1_Load(object sender, EventArgs e)
        {
            this.Visible = false;

            this.ControlBox = false;
            this.FormBorderStyle = FormBorderStyle.None;
            this.TransparencyKey = Color.Cyan;
            this.BackColor = Color.Cyan;

            // Start with the Safety skin.
            cboSkin.SelectedIndex = 0;

            this.Visible = true;
        }

        // Switch skins.
        private void cboSkin_SelectedIndexChanged(object sender, EventArgs e)
        {
            string dir = Application.StartupPath;
            int pos = dir.LastIndexOf(@"\");
            pos = dir.LastIndexOf(@"\", pos - 1);
            dir = dir.Substring(0, pos + 1);

            switch (cboSkin.SelectedIndex)
            {
                case 0:
                    LoadSkinFromDirectory(dir + "Safety");
                    lblTitle.Top = panTopContainer.Height - lblTitle.Height - 14;
                    lblTitle.ForeColor = Color.Yellow;
                    break;
                case 1:
                    LoadSkinFromDirectory(dir + "Rivets");
                    lblTitle.Top = panTopContainer.Height - lblTitle.Height - 4;
                    lblTitle.ForeColor = Color.White;
                    break;
                case 2:
                    LoadSkinFromDirectory(dir + "Pipes");
                    lblTitle.Top = panTopContainer.Height - lblTitle.Height - 8;
                    lblTitle.ForeColor = Color.White;
                    break;
            }
        }

        // Load skin images from the files in a directory.
        private void LoadSkinFromDirectory(string dir)
        {
            this.Visible = false;

            if (!dir.EndsWith("/")) dir += "/";
            picLeft.BackgroundImage = LoadImage(dir + "Left.png");
            picRight.BackgroundImage = LoadImage(dir + "Right.png");
            picBottom.BackgroundImage = LoadImage(dir + "Bottom.png");
            picLowerRight.Image = LoadImage(dir + "LowerRight.png");
            picLowerLeft.Image = LoadImage(dir + "LowerLeft.png");
            picUpperLeft.Image = LoadImage(dir + "UpperLeft.png");
            picUpperRight.Image = LoadImage(dir + "UpperRight.png");
            panTop.BackgroundImage = LoadImage(dir + "Top.png");
            panMiddle.BackgroundImage = LoadImage(dir + "Middle.png");

            if (picLeft.BackgroundImage != null) picLeft.Width = picLeft.BackgroundImage.Width;
            if (picRight.BackgroundImage != null) picRight.Width = picRight.BackgroundImage.Width;

            CloseButtonUp = LoadImage(dir + "CloseUp.png");
            CloseButtonDown = LoadImage(dir + "CloseDown.png");

            if (picBottom.BackgroundImage != null) panBottomContainer.Height = picBottom.BackgroundImage.Height;
            if (picUpperLeft.Image != null) panTopContainer.Height = picUpperLeft.Image.Height;

            picClose.Image = CloseButtonUp;
            picClose.Left = (picUpperLeft.Width - picClose.Width) / 2;
            picClose.Top = (picUpperLeft.Height - picClose.Height) / 2;

            lblTitle.Top = panTopContainer.Height - lblTitle.Height - 2;

            this.Visible = true;
        }

        // Load a skin image with error handling.
        private Bitmap LoadImage(string file)
        {
            try { return new Bitmap(file); }
            catch { return null; }
        }

        // Only close if the user clicked the Close button.
        private bool CloseOk = false;
        private void Form1_FormClosing(object sender, FormClosingEventArgs e)
        {
            e.Cancel = !CloseOk;
        }

        // Close the form.
        private void picClose_Click(object sender, EventArgs e)
        {
            CloseOk = true;
            Close();
        }

        // On mouse down, start moving the form.
        private void panTop_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(panTop, HT_CAPTION);
        }

        // On mouse down, start moving the form.
        private void lblTitle_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(lblTitle, HT_CAPTION);
        }

        // Resize with various controls.
        private void picLowerRight_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(sender as Control, HT_BOTTOMRIGHT);
        }
        private void picLowerLeft_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(sender as Control, HT_BOTTOMLEFT);
        }
        private void picUpperLeft_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(sender as Control, HT_TOPLEFT);
        }
        private void picUpperRight_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(sender as Control, HT_TOPRIGHT);
        }
        private void picRight_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(sender as Control, HT_RIGHT);
        }
        private void picBottom_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(sender as Control, HT_BOTTOM);
        }
        private void picLeft_MouseDown(object sender, MouseEventArgs e)
        {
            ProcessMouseDownMessage(sender as Control, HT_LEFT);
        }

        private const int WM_NCLBUTTONDOWN = 0xA1;
        private const int HT_CAPTION = 2;
        private const int HT_BOTTOM = 15;
        private const int HT_BOTTOMLEFT = 16;
        private const int HT_BOTTOMRIGHT = 17;
        private const int HT_LEFT = 10;
        private const int HT_RIGHT = 11;
        private const int HT_TOP = 12;
        private const int HT_TOPLEFT = 13;
        private const int HT_TOPRIGHT = 14;

        // Let the user move the form.
        private void ProcessMouseDownMessage(Control ctl, int wParam)
        {
            ctl.Capture = false;

            Message msg = Message.Create(this.Handle,
                WM_NCLBUTTONDOWN, (IntPtr)wParam, IntPtr.Zero);
            WndProc(ref msg);
        }

        // The Close button's images.
        private Image CloseButtonDown, CloseButtonUp;

        // Make the close button act like a button.
        private bool CloseButtonIsDown = false;
        private void picClose_MouseDown(object sender, MouseEventArgs e)
        {
            CloseButtonIsDown = true;
            picClose.Image = CloseButtonDown;
        }
        private void picClose_MouseUp(object sender, MouseEventArgs e)
        {
            picClose.Image = CloseButtonUp;
            CloseButtonIsDown = false;
        }
        private void picClose_MouseMove(object sender, MouseEventArgs e)
        {
            if (CloseButtonIsDown)
            {
                Rectangle rect = new Rectangle(0, 0, picClose.Width, picClose.Height);
                if (rect.Contains(e.Location))
                {
                    picClose.Image = CloseButtonDown;
                }
                else
                {
                    picClose.Image = CloseButtonUp;
                }
            }
        }
    }
}
